@@ -6,14 +6,14 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.002_001'; # TRIAL VERSION
+our $VERSION = '0.007_001'; # TRIAL VERSION
 
 use Moo;
 use Config;
 use JSON;
 use Types::Standard qw(Int Str);
 use File::Which qw(which);
-use IPC::Run qw(timeout);
+use IPC::Run qw();
 use namespace::autoclean;
 
 use constant KALEIDO => 'kaleido';
@@ -25,32 +25,35 @@ has timeout => (
     default => 30,
 );
 
-has all_formats => (
-    is       => 'ro',
-    init_arg => 0,
-    default  => sub { [] },
-);
-
-has scope_name => (
-    is       => 'ro',
-    init_arg => 0,
-);
-
-has scope_flags => (
-    is       => 'ro',
-    init_arg => 0,
-    default  => sub { [] },
-);
-
 has base_args => (
     is       => 'ro',
     init_arg => 0,
     default  => sub { [] },
 );
 
+has _default_chromium_args => (
+    is      => 'ro',
+    default => sub {
+        [
+            qw(
+              --disable-gpu
+              --allow-file-access-from-files
+              --disable-breakpad
+              --disable-dev-shm-usage
+            )
+        ];
+    }
+);
+
+has disable_gpu => (
+    is      => 'ro',
+    default => 1,
+);
+
 has _stall_timeout => (
-    is      => 'lazy',
-    builder => sub { timeout( $_[0]->timeout, name => 'stall timeout' ) },
+    is => 'lazy',
+    builder =>
+      sub { IPC::Run::timeout( $_[0]->timeout, name => 'stall timeout' ) },
 );
 
 has _h => ( is => 'rw' );
@@ -62,6 +65,11 @@ has _ios => (
     },
 );
 
+# class attributes
+sub all_formats { [] }
+sub scope_name  { "" }
+sub scope_flags { [] }
+
 sub DEMOLISH {
     my ($self) = @_;
     $self->shutdown_kaleido;
@@ -78,13 +86,24 @@ sub kaleido_args {
     my ($self) = @_;
 
     my @args = @{ $self->base_args };
+    unless ( $self->disable_gpu ) {
+        @args = grep { $_ ne '--disable-gpu' } @args;
+    }
+
     no strict 'refs';
     push @args, map {
         my $val = $self->$_;
         if ( defined $val ) {
             my $flag = $_;
             $flag =~ s/_/-/g;
-            "--$flag=$val";
+
+            # too bad Perl does not have a core boolean type..
+            if ( ref($val) =~ /^(JSON::.*::Boolean|boolean)$/ and $val ) {
+                "--$flag";
+            }
+            else {
+                "--$flag=$val";
+            }
         }
         else {
             ();
@@ -135,7 +154,9 @@ sub do_transform {
     my ( $self, $data ) = @_;
 
     $self->ensure_kaleido;
-    $self->_ios->{in} .= encode_json($data) . "\n";
+
+    my $json = JSON->new->allow_blessed(1)->convert_blessed(1);
+    $self->_ios->{in} .= $json->encode($data) . "\n";
     $self->_stall_timeout->start;
     my $resp = $self->_get_kaleido_out;
     return $resp;
@@ -246,7 +267,7 @@ Chart::Kaleido - Base class for Chart::Kaleido
 
 =head1 VERSION
 
-version 0.002_001
+version 0.007_001
 
 =head1 SYNOPSIS
 
@@ -258,7 +279,7 @@ version 0.002_001
     END_OF_TEXT
 
     my $kaleido = Chart::Plotly::Kaleido->new();
-    $kaleido->save( file => "foo.png", plotly => decode_json($data),
+    $kaleido->save( file => "foo.png", plot => $data,
                     widht => 1024, height => 768 );
 
 =head1 DESCRIPTION
@@ -282,9 +303,15 @@ L<Alien::Plotly::Kaleido>
 
 Stephan Loyd <sloyd@cpan.org>
 
+=head1 CONTRIBUTOR
+
+=for stopwords Gabor Szabo
+
+Gabor Szabo <gabor@szabgab.com>
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Stephan Loyd.
+This software is copyright (c) 2020-2021 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
