@@ -6,7 +6,7 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.002_001'; # TRIAL VERSION
+our $VERSION = '0.007_001'; # TRIAL VERSION
 
 use Moo;
 extends 'Chart::Kaleido';
@@ -25,60 +25,82 @@ use namespace::autoclean;
 my @text_formats = qw(svg json eps);
 
 
-has '+all_formats' =>
-  ( default => sub { [qw(png jpg jpeg webp svg pdf eps json)] } );
-
-has '+scope_name' => ( default => 'plotly' );
-
-has '+scope_flags' =>
-  ( default => sub { [qw(plotlyjs mathjax topojson mapbox_access_token)] }, );
-
-has '+base_args' =>
-  ( default => sub { [ qw(plotly --disable-gpu) ] } );
-
+my $default_plotlyjs = sub {
+    my $plotlyjs;
+    eval {
+        $plotlyjs = File::ShareDir::dist_file( 'Chart-Plotly',
+            'plotly.js/plotly.min.js' );
+    };  
+    return $plotlyjs;
+};
 
 has plotlyjs => (
     is      => 'ro',
-    isa     => (Str | Undef),
+    isa     => ( Str->plus_coercions( Undef, $default_plotlyjs ) | Undef ),
+    default => $default_plotlyjs,
     coerce  => 1,
-    builder => sub {
-        my $plotlyjs;
-        eval {
-            $plotlyjs = File::ShareDir::dist_file( 'Chart-Plotly',
-                'plotly.js/plotly.min.js' );
-        };
-        return $plotlyjs;
-    },
 );
 
 has [qw(mathjax topojson)] => (
     is     => 'ro',
-    isa    => (Str | Undef),
+    isa    => ( Str | Undef ),
     coerce => 1,
 );
 
 has mapbox_access_token => (
     is  => 'ro',
-    isa => (Str | Undef),
+    isa => ( Str | Undef ),
 );
 
 
+my $PositiveInt = Int->where( sub { $_ > 0 } );
+
+has default_format => (
+    is      => 'ro',
+    isa     => Str,
+    default => 'png',
+);
+
+has default_width => (
+    is      => 'ro',
+    isa     => $PositiveInt,
+    default => 700,
+);
+
+has default_height => (
+    is      => 'ro',
+    isa     => $PositiveInt,
+    default => 500,
+);
+
+has '+base_args' => (
+    default => sub {
+        my $self = shift;
+        [ "plotly", @{ $self->_default_chromium_args }, "--no-sandbox" ];
+    }
+);
+
+sub all_formats { [qw(png jpg jpeg webp svg pdf eps json)] }
+sub scope_name  { 'plotly' }
+sub scope_flags { [qw(plotlyjs mathjax topojson mapbox_access_token)] }
+
+
 sub transform {
     my $self = shift;
     state $check = compile_named_oo(
     #<<< no perltidy
-        plotly => ( HashRef | InstanceOf["Chart::Plotly::Plot"] ),
-        format => Optional[Str],
-        width  => Int,
-        height => Int,
+        plot   => ( HashRef | InstanceOf["Chart::Plotly::Plot"] ),
+        format => Optional[Str], { default => sub { $self->default_format } },
+        width  => $PositiveInt, { default => sub { $self->default_width } },
+        height => $PositiveInt, { default => sub { $self->default_height} },
         scale  => Num, { default => 1 },
     #>>>
     );
     my $arg = $check->(@_);
-    my $plotly =
-        $arg->plotly->$_isa('Chart::Plotly::Plot')
-      ? $arg->plotly->TO_JSON
-      : $arg->plotly;
+    my $plot =
+        $arg->plot->$_isa('Chart::Plotly::Plot')
+      ? $arg->plot->TO_JSON
+      : $arg->plot;
     my $format = lc( $arg->format );
 
     unless ( grep { $_ eq $format } @{ $self->all_formats } ) {
@@ -91,8 +113,10 @@ sub transform {
         width  => $arg->width,
         height => $arg->height,
         scale  => $arg->scale,
-        data   => $plotly,
+        data   => $plot,
     };
+
+    local *PDL::TO_JSON = sub { $_[0]->unpdl };
     my $resp = $self->do_transform($data);
     if ( $resp->{code} != 0 ) {
         die $resp->{message};
@@ -108,11 +132,11 @@ sub save {
     my $self = shift;
     state $check = compile_named_oo(
     #<<< no perltidy
-        file    => Path,
-        plotly => ( HashRef | InstanceOf["Chart::Plotly::Plot"] ),
+        file   => Path,
+        plot   => ( HashRef | InstanceOf["Chart::Plotly::Plot"] ),
         format => Optional[Str],
-        width  => Int,
-        height => Int,
+        width  => $PositiveInt, { default => sub { $self->default_width } },
+        height => $PositiveInt, { default => sub { $self->default_height} },
         scale  => Num, { default => 1 },
     #>>>
     );
@@ -127,7 +151,7 @@ sub save {
     $format = lc($format);
 
     my $img = $self->transform(
-        plotly => $arg->plotly,
+        plot   => $arg->plot,
         format => $format,
         width  => $arg->width,
         height => $arg->height,
@@ -150,19 +174,30 @@ Chart::Kaleido::Plotly - Export static images of Plotly charts using Kaleido
 
 =head1 VERSION
 
-version 0.002_001
+version 0.007_001
 
 =head1 SYNOPSIS
 
     use Chart::Kaleido::Plotly;
     use JSON;
 
+    my $kaleido = Chart::Kaleido::Plotly->new();
+
+    # convert a hashref
     my $data = decode_json(<<'END_OF_TEXT');
     { "data": [{"y": [1,2,1]}] }
     END_OF_TEXT
+    $kaleido->save( file => "foo.png", plot => $data,
+                    width => 1024, height => 768 );
 
-    my $kaleido = Chart::Kaleido::Plotly->new();
-    $kaleido->save( file => "foo.png", plotly => decode_json($data),
+    # convert a Chart::Plotly::Plot object
+    use Chart::Plotly::Plot;
+    my $plot = Chart::Plotly::Plot->new(
+        traces => [
+            Chart::Plotly::Trace::Scatter->new( x => [ 1 .. 5 ], y => [ 1 .. 5 ] )
+        ]
+    );
+    $kaleido->save( file => "foo.png", plot => $plot,
                     width => 1024, height => 768 );
 
 =head1 DESCRIPTION
@@ -173,10 +208,6 @@ This class wraps the "plotly" scope of plotly's kaleido command.
 
 =head2 timeout
 
-=head2 all_formats
-
-All supported formats.
-
 =head2 plotlyjs
 
 Path to plotly js file.
@@ -188,22 +219,38 @@ Default value is plotly js bundled with L<Chart::Ploly>.
 
 =head2 mapbox_access_token
 
+=head2 default_format
+
+Default is "png".
+
+=head2 default_width
+
+Default is 700.
+
+=head2 default_height
+
+Default is 500.
+
 =head1 METHODS
 
 =head2 transform
 
-    transform(( HashRef | InstanceOf["Chart::Plotly::Plot"] ) :$plotly,
-              Optional[Str] :$format,
-              Int :$width, Int :$height, Num :$scale=1)
+    transform(( HashRef | InstanceOf["Chart::Plotly::Plot"] ) :$plot,
+              Str :$format=$self->default_format,
+              PositiveInt :$width=$self->default_width,
+              PositiveInt :$height=$self->default_height,
+              Num :$scale=1)
 
 Returns raw image data.
 
 =head2 save
 
     save(:$file,
-         ( HashRef | InstanceOf["Chart::Plotly::Plot"] ) :$plotly,
+         ( HashRef | InstanceOf["Chart::Plotly::Plot"] ) :$plot,
          Optional[Str] :$format,
-         Int :$width, Int :$height, Num :$scale=1)
+         PositiveInt :$width=$self->default_width,
+         PositiveInt :$height=$self->default_height,
+         Num :$scale=1)
 
 Save static image to file.
 
@@ -221,7 +268,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Stephan Loyd.
+This software is copyright (c) 2020-2021 by Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
